package eu.jonahbauer.wizard.common.util;

import com.google.gson.Gson;
import com.google.gson.TypeAdapter;
import com.google.gson.TypeAdapterFactory;
import com.google.gson.reflect.TypeToken;
import com.google.gson.typeadapters.RuntimeTypeAdapterFactory;
import org.jetbrains.annotations.Nullable;

import java.util.Locale;

public final class SealedClassTypeAdapterFactory<T> implements TypeAdapterFactory {
    private final RuntimeTypeAdapterFactory<T> factory;

    public static <T> SealedClassTypeAdapterFactory<T> of(Class<T> clazz) {
        return new SealedClassTypeAdapterFactory<>(clazz, null);
    }

    public static <T> SealedClassTypeAdapterFactory<T> of(Class<T> clazz, @Nullable String suffix) {
        return new SealedClassTypeAdapterFactory<>(clazz, suffix);
    }

    private SealedClassTypeAdapterFactory(Class<T> clazz, @Nullable String suffix) {
        factory = RuntimeTypeAdapterFactory.of(clazz);
        for (Class<?> subclass : clazz.getPermittedSubclasses()) {
            String name = subclass.getSimpleName();

            // remove suffix
            if (suffix != null) {
                if (name.endsWith(suffix)) name = name.substring(0, name.length() - suffix.length());
            }

            // transform camelCast to snake_case
            name = name.replaceAll("([a-z])([A-Z]+)", "$1_$2").toLowerCase(Locale.ROOT);

            factory.registerSubtype(subclass.asSubclass(clazz), name);
        }
    }

    @Override
    public <S> TypeAdapter<S> create(Gson gson, TypeToken<S> type) {
        return factory.create(gson, type);
    }
}
