package eu.jonahbauer.wizard.core.machine.states.trick;

import eu.jonahbauer.wizard.core.machine.Game;
import eu.jonahbauer.wizard.core.machine.states.GameData;
import eu.jonahbauer.wizard.core.machine.states.round.FinishingRound;
import eu.jonahbauer.wizard.common.messages.observer.PredictionMessage;
import eu.jonahbauer.wizard.common.messages.observer.UserInputMessage;
import eu.jonahbauer.wizard.common.messages.player.PlayerMessage;
import eu.jonahbauer.wizard.common.messages.player.PredictMessage;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import static eu.jonahbauer.wizard.core.machine.states.GameData.*;
import static eu.jonahbauer.wizard.common.messages.observer.UserInputMessage.Action.*;

public final class ChangingPrediction extends TrickState {
    private transient final int oldPrediction;

    public ChangingPrediction(GameData data) {
        super(data);
        oldPrediction = get(PREDICTIONS).get(get(CURRENT_PLAYER));
    }

    @Override
    public void onEnter(Game game) {
        game.notify(new UserInputMessage(get(CURRENT_PLAYER), CHANGE_PREDICTION, getTimeout(game, true)));
        timeout(game);
    }

    @Override
    public void onMessage(Game game, UUID player, PlayerMessage message) {
        if (get(CURRENT_PLAYER).equals(player) && message instanceof PredictMessage predictMessage) {
            checkPrediction(predictMessage.getPrediction());
            transition(game, predictMessage.getPrediction());
        } else {
            super.onMessage(game, player, message);
        }
    }

    @Override
    public void onTimeout(Game game) {
        transition(game, oldPrediction + 1);
    }

    private void checkPrediction(int prediction) {
        if (prediction < 0) {
            throw new IllegalArgumentException("Prediction must be greater than or equal to 0.");
        } else if (Math.abs(prediction - oldPrediction) != 1) {
            throw new IllegalArgumentException("Prediction must differ from your old prediction by exactly one.");
        }
    }

    private void transition(Game game, int prediction) {
        game.notify(new PredictionMessage(get(CURRENT_PLAYER), prediction));

        // add prediction
        var predictions = new HashMap<>(get(PREDICTIONS));
        predictions.put(get(CURRENT_PLAYER), prediction);

        GameData data = getData().with(
                PREDICTIONS, Map.copyOf(predictions)
        );

        boolean hasNextTrick = get(TRICK) < get(ROUND);
        if (hasNextTrick) {
            transition(game, new StartingTrick(data.with(TRICK, get(TRICK) + 1)));
        } else {
            transition(game, new FinishingRound(data));
        }
    }
}
