package eu.jonahbauer.wizard.core.model;

import eu.jonahbauer.wizard.core.model.deck.Decks;
import lombok.experimental.UtilityClass;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.NoSuchElementException;

@UtilityClass
public class Configurations {
    private static final Map<String, Configuration> CONFIGURATIONS = new HashMap<>();

    public static final Configuration DEFAULT = register("DEFAULT", new Configuration(
            Decks.DEFAULT,
            true,
            10 * 60 * 1000
    ));

    public static final Configuration DEFAULT_PM1 = register("DEFAULT_PM1", new Configuration(
            Decks.DEFAULT,
            false,
            10 * 60 * 1000
    ));

    public static final Configuration ANNIVERSARY_2016 = register("ANNIVERSARY_2016", new Configuration(
            Decks.ANNIVERSARY_2016,
            true,
            10 * 60 * 1000
    ));

    public static final Configuration ANNIVERSARY_2016_PM1 = register("ANNIVERSARY_2016_PM1", new Configuration(
            Decks.ANNIVERSARY_2016,
            false,
            10 * 60 * 1000
    ));

    public static final Configuration ANNIVERSARY_2021 = register("ANNIVERSARY_2021", new Configuration(
            Decks.ANNIVERSARY_2021,
            true,
            10 * 60 * 1000
    ));

    public static final Configuration ANNIVERSARY_2021_PM1 = register("ANNIVERSARY_2021_PM1", new Configuration(
            Decks.ANNIVERSARY_2021,
            false,
            10 * 60 * 1000
    ));

    @Contract("_,_ -> param2")
    private static Configuration register(@NotNull String name, @NotNull Configuration configuration) {
        if (CONFIGURATIONS.putIfAbsent(name.toUpperCase(Locale.ROOT), configuration) != null) {
            throw new IllegalArgumentException("Name already taken.");
        }
        return configuration;
    }

    @NotNull
    public static Configuration get(@NotNull String name) {
        var out = CONFIGURATIONS.get(name.toUpperCase(Locale.ROOT));
        if (out == null) throw new NoSuchElementException("Configuration with name '" + name + "' does not exist.");
        return out;
    }
}
